"""
by www.ixi-software.net
license :GPL

version 0.1

Description:
Very simple widgets server based on wxPython. It defines a set of classes that send OSC messages.
It allows to control sound engines in ChucK or Supercollider via OSC.
Check example.py file

Dependencies:
python - www.python.org
wxpython - www.wxpython.org
simpleosc - www.ixi-software.net/content/body_backyard_python.html

wish list:
control via OSC o allow to create widgets from ChuCK or Supercollider etc ...
scroll numbers
plot waves/graphs with wx.lib.plot.py ?
drag'n'dop area
color canvas to structure
# file/folder import dialogue
bangs and toogle buttons to have some visual difference
implement labels on the widgets that dont have them built in
explore custom widgets
"""


import wx
import osc
import time


class OSCWidget(object):
    """ abstract widget base class that sends via OSC its value when interacted. 
    """
    def __init__(self, frame, oscaddress):
        self.oscaddress = oscaddress
        self.intEventList = [ wx.EVT_TOGGLEBUTTON.evtType[0], wx.EVT_BUTTON.evtType[0],
                              wx.EVT_COMMAND_SCROLL.evtType[0],  wx.EVT_SPIN.evtType[0]
                              ]
##        self.start()
##
##    def start(self): pass # to extend by user

    def action(self, e):
        if e.GetEventType() in self.intEventList :
            data = e.GetInt()
        else :
            data = str(e.GetString())
            
##        print data, e.GetInt(), e.GetString()
        osc.sendMsg(self.oscaddress, [data], App.ip, App.outPort)
        

class Label(wx.StaticText) :
    def __init__(self, frame, label="toogle", pos=wx.DefaultPosition, size=wx.DefaultSize):
        wx.StaticText.__init__(self, frame, -1, label, pos, size)


class Toggle(wx.ToggleButton, OSCWidget):
    def __init__(self, frame, oscaddress, label="X", pos=wx.DefaultPosition, size=wx.DefaultSize):
        wx.ToggleButton.__init__(self, frame, -1, label, pos, size)
        OSCWidget.__init__(self, frame, oscaddress)
        frame.Bind(wx.EVT_TOGGLEBUTTON, self.action, self)


class Button(wx.Button, OSCWidget):
    def __init__(self, frame, oscaddress, label="O", pos=wx.DefaultPosition, size=wx.DefaultSize):
        wx.Button.__init__(self, frame, -1, label, pos, size)
        OSCWidget.__init__(self, frame, oscaddress)
        frame.Bind(wx.EVT_BUTTON, self.action, self)


class Slider(wx.Slider, OSCWidget):
    def __init__(self, frame, oscaddress, label="", pos=wx.DefaultPosition,
                 size=wx.DefaultSize, range=(0, 127), style='horizontal'):
        if style == 'vertical' or style == 'v':
            style = wx.SL_VERTICAL | wx.SL_LABELS
            if size == wx.DefaultSize : size = (-1, 127)
        else:
            style = wx.SL_HORIZONTAL | wx.SL_LABELS
            if size == wx.DefaultSize : size = (127, -1)
                
        wx.Slider.__init__(self, frame, -1, value=25, minValue=range[0], maxValue=range[1],
                           pos=pos, size=size, style=style )
        OSCWidget.__init__(self, frame, oscaddress)
        frame.Bind(wx.EVT_COMMAND_SCROLL_THUMBTRACK, self.action, self)


class Choice(wx.Choice, OSCWidget):
    def __init__(self, frame, oscaddress, label="", pos=wx.DefaultPosition, size=wx.DefaultSize, choices = ['zero', 'one', 'two'] ):
        wx.Choice.__init__(self, frame, -1, pos=pos, size=size, choices = choices)
        OSCWidget.__init__(self, frame, oscaddress)
        frame.Bind(wx.EVT_CHOICE, self.action, self)


class RadioBox(wx.RadioBox, OSCWidget):
    def __init__(self, frame, oscaddress, label="", pos=wx.DefaultPosition, size=wx.DefaultSize,
                 choices = ['zero', 'one', 'two'] , selected=0):
        wx.RadioBox.__init__(self, frame, -1, label, pos=pos, size=size, choices = choices ,
                             majorDimension = 1, style=wx.RA_SPECIFY_COLS )
        OSCWidget.__init__(self, frame, oscaddress)
        frame.Bind(wx.EVT_RADIOBOX, self.action, self)
        self.SetSelection(selected) # which one is selected by default
        

class Number(wx.TextCtrl, OSCWidget):
    def __init__(self, frame, oscaddress, label="", pos=wx.DefaultPosition, size=(30, 25),
                 value=0, range=(0, 127) ):
        wx.TextCtrl.__init__(self, frame, -1, str(value), pos=pos, size=size)
        OSCWidget.__init__(self, frame, oscaddress)

        spin = wx.SpinButton(frame, -1, (pos[0] - 27, pos[1] ), self.GetSize(), wx.SP_VERTICAL)
        spin.SetRange(range[0], range[1])
        spin.SetValue(value)

        frame.Bind(wx.EVT_SPIN, self.action, spin)
        
    def action(self, e) : # extends superclass action
        OSCWidget.action(self, e)
        self.SetValue(str(e.GetPosition()))

###################################################################

class FpsTimer(wx.Timer):
    def __init__(self, fps, f):
        wx.Timer.__init__(self)
        self.Start(1000./fps)
        self.f = f # function to call on notify

    def Notify(self): self.f()



class App(wx.App):

    outPort = 0
    inPort = 0
    ip = 0

    def __init__(self, name='app', pos=wx.DefaultPosition, size=wx.DefaultSize, fps=15, ip='127.0.0.1',
                 outPort=9001, inPort=9000):
        wx.App.__init__(self, 0)

        self.connected = 0
        self.fps = fps
        
        # OSC
        App.ip = ip
        App.outPort = outPort
        App.inPort = inPort
        osc.init()
        self.oscIn = osc.createListener(port=inPort) # in this case just using one socket
        osc.bind(self.status, "/connected")
        
        # contructing the frame and the widgets
        self.frame = wx.Frame(None, -1, name, pos, size)
        self.doFrame() # users place their code here. define the widgets
        self.frame.CreateStatusBar(1,0)
        self.frame.SetStatusText("OSC status : not connected")
        self.frame.Show(True)
        self.SetTopWindow(self.frame)
        self.t = FpsTimer(self.fps, self.step) # for the incomming osc loop

        osc.sendMsg('/connected', [1], port=App.outPort) # all ready, send notification

        # wait for a sucessful connected message from sound server
        past = time.time()
        while not self.connected : # wait 5 secs for welcome message
            time.sleep(0.1)
            osc.getOSC(self.oscIn)
            if time.time() - past > 5 : 
                self.frame.SetStatusText('I didnt get a connected message from sound server')
                break
            
        self.MainLoop() # main loop
        
        osc.sendMsg('/connected', [0], port=App.outPort) # app has exited the main loop, quiting
        

    def step(self):
        osc.getOSC(self.oscIn) # check for incoming msgs

    def status(self, *msg):
        print "conneted msg received from sound server"
        self.connected = msg[0][2]
        if self.connected :
            s = "OSC status : ok. Receiving %i . Sending %s, %i" % (App.inPort, App.ip, App.outPort)
            self.frame.SetStatusText(s)
